@@ -1,10 +1,50 @@
 #![cfg(target_pointer_width = "64")]
 
 use crate::{broadword, darray::DArrayIndex, BitVector};
+use anyhow::{anyhow, Result};
 use serde::{Deserialize, Serialize};
 
-/// Compressed monotone sequence through Elias-Fano encoding.
-///This is a yet another Rust port of [succinct::elias_fano](https://github.com/ot/succinct/blob/master/elias_fano.hpp).
+/// Compressed monotone increasing sequence through Elias-Fano encoding.
+///
+/// [`EliasFano`] implements an Elias-Fano representation for monotone increasing sequences.
+/// When a sequence stores $`n`$ integers from $`[0, u-1]`$, this representation takes $`n \lceil \log_2 \frac{u}{n} \rceil + 2n`$ bits of space.
+/// That is, a sparse sequence can be stored in a very compressed space.
+///
+/// This is a yet another Rust port of [succinct::elias_fano](https://github.com/ot/succinct/blob/master/elias_fano.hpp).
+///
+/// # Example
+///
+/// ```
+/// use sucds::{EliasFano, EliasFanoBuilder};
+///
+/// let mut b = EliasFanoBuilder::new(10, 4).unwrap();
+/// b.push(2).unwrap();
+/// b.push(3).unwrap();
+/// b.push(6).unwrap();
+/// b.push(9).unwrap();
+///
+/// let ef = EliasFano::new(b, true);
+/// assert_eq!(ef.select(1), 3);
+/// assert_eq!(ef.select(3), 9);
+///
+/// assert_eq!(ef.rank(5), 2);
+/// assert_eq!(ef.rank(10), 4);
+///
+/// assert_eq!(ef.predecessor(5), Some(3));
+/// assert_eq!(ef.predecessor(1), None);
+///
+/// assert_eq!(ef.successor(4), Some(6));
+/// assert_eq!(ef.successor(10), None);
+/// ```
+///
+/// # References
+///
+///  - P. Elias, "Efficient storage and retrieval by content and address of static files,"
+///    Journal of the ACM, 1974.
+///  - R. Fano, "On the number of bits required to implement an associative memory,"
+///    Memorandum 61. Computer Structures Group, Project MAC, MIT, 1971.
+///  - D. Okanohara, and K. Sadakane, "Practical Entropy-Compressed Rank/Select Dictionary,"
+///    In ALENEX, 2007.
 #[derive(Serialize, Deserialize)]
 pub struct EliasFano {
     high_bits: BitVector,
@@ -28,17 +68,17 @@ impl EliasFano {
     /// ```
     /// use sucds::{EliasFano, EliasFanoBuilder};
     ///
-    /// let mut b = EliasFanoBuilder::new(10, 4);
-    /// b.push(2);
-    /// b.push(3);
-    /// b.push(6);
-    /// b.push(10);
+    /// let mut b = EliasFanoBuilder::new(10, 4).unwrap();
+    /// b.push(2).unwrap();
+    /// b.push(3).unwrap();
+    /// b.push(6).unwrap();
+    /// b.push(9).unwrap();
     ///
     /// let ef = EliasFano::new(b, false);
     /// assert_eq!(ef.select(0), 2);
     /// assert_eq!(ef.select(1), 3);
     /// assert_eq!(ef.select(2), 6);
-    /// assert_eq!(ef.select(3), 10);
+    /// assert_eq!(ef.select(3), 9);
     /// ```
     pub fn new(b: EliasFanoBuilder, with_rank_index: bool) -> Self {
         let high_bits_d1 = DArrayIndex::new(&b.high_bits, true);
@@ -64,18 +104,22 @@ impl EliasFano {
     /// - `bits`: List of bits.
     /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
     ///
+    /// # Errors
+    ///
+    /// `anyhow::Error` will be returned if `bits` contains no set bit.
+    ///
     /// # Examples
     ///
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true], true);
+    /// let ef = EliasFano::from_bits(&[true, false, false, true], true).unwrap();
     /// assert_eq!(ef.rank(2), 1);
     /// assert_eq!(ef.select(1), 3);
     /// assert_eq!(ef.predecessor(2), Some(0));
     /// assert_eq!(ef.successor(1), Some(3));
     /// ```
-    pub fn from_bits<'a, I>(bits: I, with_rank_index: bool) -> Self
+    pub fn from_bits<'a, I>(bits: I, with_rank_index: bool) -> Result<Self>
     where
         I: IntoIterator<Item = &'a bool>,
     {
@@ -89,51 +133,59 @@ impl EliasFano {
     /// - `bv`: Input bit vector.
     /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
     ///
+    /// # Errors
+    ///
+    /// `anyhow::Error` will be returned if `bv` contains no set bit.
+    ///
     /// # Examples
     ///
     /// ```
     /// use sucds::{EliasFano, BitVector};
     ///
     /// let bv = BitVector::from_bits(&[true, false, false, true]);
-    /// let ef = EliasFano::from_bitvec(&bv, true);
+    /// let ef = EliasFano::from_bitvec(&bv, true).unwrap();
     /// assert_eq!(ef.rank(2), 1);
     /// assert_eq!(ef.select(1), 3);
     /// assert_eq!(ef.predecessor(2), Some(0));
     /// assert_eq!(ef.successor(1), Some(3));
     /// ```
-    pub fn from_bitvec(bv: &BitVector, with_rank_index: bool) -> Self {
+    pub fn from_bitvec(bv: &BitVector, with_rank_index: bool) -> Result<Self> {
         let n = bv.len();
         let m = (0..bv.num_words())
             .into_iter()
             .fold(0, |acc, i| acc + broadword::popcount(bv.get_word(i)));
-        let mut b = EliasFanoBuilder::new(n, m);
+        let mut b = EliasFanoBuilder::new(n, m)?;
         for i in 0..n {
             if bv.get_bit(i) {
-                b.push(i);
+                b.push(i)?;
             }
         }
-        Self::new(b, with_rank_index)
+        Ok(Self::new(b, with_rank_index))
     }
 
-    /// Searches the `n`-th iteger.
+    /// Searches the `k`-th iteger.
     ///
     /// # Arguments
     ///
-    /// - `n`: Select query.
+    /// - `k`: Select query.
+    ///
+    /// # Complexity
+    ///
+    /// - Constant
     ///
     /// # Examples
     ///
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true], true);
+    /// let ef = EliasFano::from_bits(&[true, false, false, true], true).unwrap();
     /// assert_eq!(ef.select(0), 0);
     /// assert_eq!(ef.select(1), 3);
     /// ```
     #[inline(always)]
-    pub fn select(&self, n: usize) -> usize {
-        ((self.high_bits_d1.select(&self.high_bits, n) - n) << self.low_len)
-            | self.low_bits.get_bits(n * self.low_len, self.low_len)
+    pub fn select(&self, k: usize) -> usize {
+        ((self.high_bits_d1.select(&self.high_bits, k) - k) << self.low_len)
+            | self.low_bits.get_bits(k * self.low_len, self.low_len)
     }
 
     /// Counts the number of integers less than `pos`.
@@ -142,12 +194,16 @@ impl EliasFano {
     ///
     /// - `pos`: Rank query.
     ///
+    /// # Complexity
+    ///
+    /// - $`O(\log \frac{u}{n})`$
+    ///
     /// # Examples
     ///
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true], true);
+    /// let ef = EliasFano::from_bits(&[true, false, false, true], true).unwrap();
     /// assert_eq!(ef.rank(1), 1);
     /// assert_eq!(ef.rank(2), 1);
     /// assert_eq!(ef.rank(3), 1);
@@ -155,10 +211,9 @@ impl EliasFano {
     /// ```
     #[inline(always)]
     pub fn rank(&self, pos: usize) -> usize {
-        debug_assert!(pos <= self.universe());
         debug_assert!(self.high_bits_d0.is_some());
 
-        if pos == self.universe() {
+        if pos > self.universe() {
             return self.len();
         }
 
@@ -189,12 +244,16 @@ impl EliasFano {
     ///
     /// - `pos`: Predecessor query.
     ///
+    /// # Complexity
+    ///
+    /// - $`O(\log \frac{u}{n})`$
+    ///
     /// # Examples
     ///
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[false, true, false, false, true], true);
+    /// let ef = EliasFano::from_bits(&[false, true, false, false, true], true).unwrap();
     /// assert_eq!(ef.predecessor(4), Some(4));
     /// assert_eq!(ef.predecessor(3), Some(1));
     /// assert_eq!(ef.predecessor(0), None);
@@ -212,12 +271,16 @@ impl EliasFano {
     ///
     /// - `pos`: Successor query.
     ///
+    /// # Complexity
+    ///
+    /// - $`O(\log \frac{u}{n})`$
+    ///
     /// # Examples
     ///
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true, false], true);
+    /// let ef = EliasFano::from_bits(&[true, false, false, true, false], true).unwrap();
     /// assert_eq!(ef.successor(0), Some(0));
     /// assert_eq!(ef.successor(1), Some(3));
     /// assert_eq!(ef.successor(4), None);
@@ -241,7 +304,7 @@ impl EliasFano {
         self.len() == 0
     }
 
-    /// Gets the max of stored integers.
+    /// Gets the (exclusive) upper bound of integers.
     #[inline(always)]
     pub const fn universe(&self) -> usize {
         self.universe
@@ -264,13 +327,18 @@ impl EliasFanoBuilder {
     ///
     /// # Arguments
     ///
-    /// - `universe`: The max of integers to be stored.
-    /// - `num_ints`: The number of integers to be stored.
-    pub fn new(universe: usize, num_ints: usize) -> Self {
-        assert_ne!(universe, 0);
-        assert_ne!(num_ints, 0);
+    /// - `universe`: The (exclusive) upper bound of integers to be stored, i.e., an integer in `[0..universe - 1]`.
+    /// - `num_ints`: The number of integers to be stored (> 0).
+    ///
+    /// # Errors
+    ///
+    /// `anyhow::Error` will be returned if the given setting is invalid.
+    pub fn new(universe: usize, num_ints: usize) -> Result<Self> {
+        if num_ints == 0 {
+            return Err(anyhow!("num_ints must not be zero"));
+        }
         let low_len = broadword::msb(universe / num_ints).unwrap_or(0);
-        Self {
+        Ok(Self {
             high_bits: BitVector::with_len((num_ints + 1) + (universe >> low_len) + 1),
             low_bits: BitVector::new(),
             universe,
@@ -278,7 +346,7 @@ impl EliasFanoBuilder {
             pos: 0,
             last: 0,
             low_len,
-        }
+        })
     }
 
     /// Pushes integer `i` at the end.
@@ -286,9 +354,32 @@ impl EliasFanoBuilder {
     /// # Arguments
     ///
     /// - `i`: Pushed integer that must be no less than the last one.
-    pub fn push(&mut self, i: usize) {
-        assert!(i >= self.last && i <= self.universe);
-        assert!(self.pos < self.num_ints);
+    ///
+    /// # Errors
+    ///
+    /// `anyhow::Error` will be returned if the input integer is invalid.
+    pub fn push(&mut self, i: usize) -> Result<()> {
+        if i < self.last {
+            return Err(anyhow!(
+                "The input integer {} must be no less than the last one {}",
+                i,
+                self.last
+            ));
+        }
+        if self.universe <= i {
+            return Err(anyhow!(
+                "The input integer {} must be less than the universe {}",
+                i,
+                self.universe
+            ));
+        }
+        if self.num_ints <= self.pos {
+            return Err(anyhow!(
+                "The number of stored integers exceeds the given upper bound {}",
+                self.num_ints
+            ));
+        }
+
         self.last = i;
         let low_mask = (1 << self.low_len) - 1;
         if self.low_len != 0 {
@@ -296,6 +387,8 @@ impl EliasFanoBuilder {
         }
         self.high_bits.set_bit((i >> self.low_len) + self.pos, true);
         self.pos += 1;
+
+        Ok(())
     }
 }
 
@@ -350,7 +443,8 @@ mod tests {
 
     #[test]
     fn test_tiny_bits() {
-        let ef = EliasFano::from_bits(&[true, false, false, true, false, true, true], false);
+        let ef =
+            EliasFano::from_bits(&[true, false, false, true, false, true, true], false).unwrap();
         assert_eq!(ef.select(0), 0);
         assert_eq!(ef.select(1), 3);
         assert_eq!(ef.select(2), 5);
@@ -361,7 +455,7 @@ mod tests {
     fn test_random_bits() {
         for seed in 0..100 {
             let bits = gen_random_bits(10000, seed);
-            let ef = EliasFano::from_bits(&bits, true);
+            let ef = EliasFano::from_bits(&bits, true).unwrap();
             test_rank_select(&bits, &ef);
             test_successor_predecessor(&bits, &ef);
         }
@@ -370,7 +464,7 @@ mod tests {
     #[test]
     fn test_serialize() {
         let bits = gen_random_bits(10000, 42);
-        let ef = EliasFano::from_bits(&bits, true);
+        let ef = EliasFano::from_bits(&bits, true).unwrap();
         let bytes = bincode::serialize(&ef).unwrap();
         let other: EliasFano = bincode::deserialize(&bytes).unwrap();
         assert_eq!(ef.len(), other.len());
